/*
  This file is part of the Linux driver for the Eltron p310c plastic card printer
    Copyright (C) 2002 Flemming Frandsen <ff-p310c@partyticket.net>
    Copyright (C) 2002 Christian Laursen <xi@borderworlds.dk>
    Based on initial work by Stian Skjelstad <stian@nixia.no>
    Materials (printer+cards+ribbons) provided by http://www.psi.no/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/
#include "pngloader.h"
#include <stdio.h>
#include <stdlib.h>
#include <png.h>

Image png_load(char *filename)
{
  FILE *fp = fopen(filename, "rb");
  if (!fp) {
      perror("png_load(): fopen failed");
      exit(-6);
  }

  /* Check the header */

  unsigned char header[10];

  fread(header, 1, 8, fp);
  int is_png = !png_sig_cmp(header, 0, 8);
  if (!is_png) {
    fprintf(stderr, "%s is not a png file\n", filename);
    exit(1);
  }

  /* Initialise the png library */

  png_structp png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING,
						(png_voidp)0,
						(png_error_ptr)0, (png_error_ptr)0);
  if (!png_ptr) {
    fprintf(stderr, "Couldn't allocate png_struct\n");
    exit(1);
  }

  png_infop info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
    fprintf(stderr, "Couldn't allocate info_struct\n");
    exit(1);
  }

  png_infop end_info = png_create_info_struct(png_ptr);
  if (!end_info) {
    png_destroy_read_struct(&png_ptr, &info_ptr,
			    (png_infopp)NULL);
    fprintf(stderr, "Couldn't allocate end info_struct\n");
    exit(1);
  }

  /* Initialise I/O */

  png_init_io(png_ptr, fp);
  png_set_sig_bytes(png_ptr, 8);

  /* Read the image */

  png_read_png(png_ptr, info_ptr, 0, NULL);

  png_bytep *row_pointers;
  row_pointers = png_get_rows(png_ptr, info_ptr);

  Image img;
  img.width = png_get_image_width(png_ptr,info_ptr);
  img.height = png_get_image_height(png_ptr,info_ptr);

  if (png_get_bit_depth(png_ptr,info_ptr) != 8) {
    fprintf(stderr, "Error loading image: %s  - only bit depth 8 is supported\n",filename);
    exit(1);
  }

  unsigned int *img_data = (unsigned int *)malloc(img.width * img.height * sizeof(int));

  int color_type = png_get_color_type(png_ptr,info_ptr);
  int alpha;
  if (color_type == PNG_COLOR_TYPE_RGB) {
    alpha = 0;
  } else if (color_type == PNG_COLOR_TYPE_RGB_ALPHA) {
    alpha = 1;
  } else {
    fprintf(stderr, "Error loading image: %s - it uses grayscale, palette or other unsupported crap\n",filename);
    exit(1);
  }

  int x, y;
  for (y = 0; y < img.height; ++y) {
    for (x = 0; x < img.width; ++x) {
      if (!alpha) {
	img_data[y * img.width + x] = (row_pointers[y][x*3]   << 16) + 
                                      (row_pointers[y][x*3+1] << 8) + 
                                      (row_pointers[y][x*3+2]);
      } else {
	img_data[y * img.width + x] = (row_pointers[y][x*4]   << 16) + 
                                      (row_pointers[y][x*4+1] << 8) + 
                                      (row_pointers[y][x*4+2]) + 
                                      (row_pointers[y][x*4+3] << 24);	
      }
    }
  }

  img.imagedata = img_data;

  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);

  return img;
}
